package MT::CuttingsApp;
use strict;

use File::Basename;

use MT::Util qw( spam_protect );
use MT::Template::Context;
use MT::Plugin::Cuttings;

my $plugin = MT::Plugin::Cuttings->new;
MT->add_plugin($plugin);

#--- tag handlers

MT::Template::Context->add_container_tag(Authors => \&authors);
map { MT::Template::Context->add_tag($_ => \&author_handler) }
  qw( AuthorDisplayName AuthorUsername AuthorEmail AuthorURL );
MT::Template::Context->add_container_tag(AuthorEntries => \&author_entries);
MT::Template::Context->add_tag(AuthorLink       => \&author_link);
MT::Template::Context->add_tag(AuthorEntryCount => \&author_entry_count);
MT::Template::Context->add_tag(AuthorArchiveURL => \&author_archive_url);

my %COLS = (
            AuthorDisplayName => 'nickname',
            AuthorUsername    => 'name',
            AuthorEmail       => 'email',
            AuthorURL         => 'url'
);

sub author_handler {
    my ($ctx, $args) = @_;
    my $tag = $ctx->stash('tag');
    my $a   = $ctx->stash('author') or return $ctx->_no_author_error;
    my $col = $COLS{$tag};
    my $str = $a->$col || '';
    $str = spam_protect($str) if $col eq 'email' && $args->{'spam_protect'};
    $str;
}

sub author_link {
    my ($ctx, $args) = @_;
    my $a = $ctx->stash('author') or return $ctx->_no_author_error;
    my $name = $a->name || '';
    my $show_email = $args->{show_email} ? 1 : 0;
    my $show_url = 1 unless exists $args->{show_url} && !$args->{show_url};
    if ($show_url && $a->url) {
        return sprintf qq(<a href="%s">%s</a>), $a->url, $name;
    } elsif ($show_email && $a->email) {
        my $str = "mailto:" . $a->email;
        $str = spam_protect($str) if $args->{'spam_protect'};
        return sprintf qq(<a href="%s">%s</a>), $str, $name;
    } else {
        return $name;
    }
}

sub authors {
    my ($ctx, $args, $cond) = @_;
    my $r       = '';
    my $builder = $ctx->stash('builder');
    my $tokens  = $ctx->stash('tokens');
    my $blog    = $ctx->stash('blog');
    my $blog_id = $ctx->stash('blog_id') || $blog->id;
    my $t       = {type => 1};                           # author, not commenter
    my $a = {'join' => ['MT::Permission', 'author_id', {blog_id => $blog_id}]};
    $a->{'sort'} = $args->{sort_by} || 'name';
    $a->{direction} = $args->{sort_order} || 'ascend';
    require MT::Author;
    my $iter = MT::Author->load_iter($t, $a);
    while (my $author = $iter->()) {
        local $ctx->{__stash}{author} = $author;
        my $out = $builder->build($ctx, $tokens, $cond)
          or return $ctx->error($builder->errstr);
        $r .= $out;
    }
    $r;
}

sub author_entries {
    my ($ctx, $args, $cond) = @_;
    my $author  = $ctx->stash('author') or return $ctx->_no_author_error;
    my $r       = '';
    my $builder = $ctx->stash('builder');
    my $tokens  = $ctx->stash('tokens');
    my $blog    = $ctx->stash('blog');
    my $blog_id = $ctx->stash('blog_id') || $blog->id;
    require MT::Entry;
    my $t = {
             blog_id   => $blog_id,
             author_id => $author->id,
             status    => MT::Entry::RELEASE()
    };
    my $a = {};
    $a->{'sort'} = $args->{sort_by} || 'created_on';
    $a->{direction} = $args->{sort_order} || 'descend';
    $a->{limit}  = $args->{lastn}  if $args->{lastn};
    $a->{offset} = $args->{offset} if $args->{offset};
    my $iter = MT::Entry->load_iter($t, $a);
    while (my $e = $iter->()) {
        local $ctx->{__stash}{entry} = $e;
        my $out = $builder->build($ctx, $tokens, $cond)
          or return $ctx->error($builder->errstr);
        $r .= $out;
    }
    $r;
}

sub author_entry_count {
    my ($ctx, $args, $cond) = @_;
    my $a       = $ctx->stash('author') or return $ctx->_no_author_error;
    my $blog    = $ctx->stash('blog');
    my $blog_id = $ctx->stash('blog_id') || $blog->id;
    require MT::Entry;
    my $count =
      MT::Entry->count(
                       {
                        blog_id   => $blog_id,
                        author_id => $a->id,
                        status    => MT::Entry::RELEASE()
                       }
      );
    $count;
}

sub author_archive_url {
    my ($ctx, $args, $cond) = @_;
    my $author  = $ctx->stash('author') or return $ctx->_no_author_error;
    my $blog    = $ctx->stash('blog');
    my $blog_id = $ctx->stash('blog_id') || $blog->id;
    my $cfg     = $plugin->config_mgr("blog:$blog_id");
    return '' unless $cfg->ArchiveTemplateID;
    my $file;
    my $c = MT::Template::Context->new;
    $c->stash('blog',    $blog);
    $c->stash('blog_id', $blog_id);
    $c->stash('author',  $author);
    my $builder        = MT::Builder->new;
    my $filename_tmpls = MT->instance->request('__cached_filename_tmpls')
      || MT->instance->request('__cached_filename_tmpls', {});
    my $tokens = $filename_tmpls->{$cfg->ArchiveTemplateID};
    unless ($tokens) {
        $tokens = $builder->compile($c, $cfg->FilenameTemplate)
          or return $blog->error($builder->errstr);
        $filename_tmpls->{$cfg->ArchiveTemplateID} = $tokens;
    }
    $c->stash('builder', $builder);
    $c->stash('tokens',  $tokens);
    defined($file = $builder->build($c, $tokens))
      or return $blog->error($builder->errstr);
    my $url = $blog->archive_url;
    $url .= '/' unless $url =~ m!/$!;
    $url . $file;
}

sub _no_author_error {
    my $tag = 'MT' . $_[0]->stash('tag');
    return
      $_[0]->error(
           MT->translate(
                  "You used an '[_1]' tag outside of the context of an author.",
                  $tag
           )
      );
}

#--- archive publishing

my $app = MT->instance;
if (ref $app eq 'MT::App::CMS')
{    # MT can't tell us what blog if any we are in at this point.
    my $opt = {
               Name => MT->translate('Rebuild Author Archives Only'),
               Code => \&rebuild_author_archives
    };
    MT::App::CMS->add_rebuild_option($opt);
}

MT->add_callback('CMSPostEntrySave', 4, $plugin, \&rebuild_entry_author);

sub rebuild_entry_author {
    my ($eh, $app, $entry) = @_;
    my $blog_id = $entry->blog_id;
    my $cfg     = $plugin->config_mgr("blog:$blog_id");
    return unless $cfg->ArchiveTemplateID;
    rebuild_author_archives(    # will errors get caught?
        $app,
        AuthorID => $entry->author_id,
        BlogID   => $blog_id
    );
}

sub rebuild_author_archives {
    my $app   = shift || MT->instance;
    my %param = @_;
    my $blog  = $param{Blog} || $app->blog;
    my $blog_id;
    unless ($blog) {
        $blog_id = $param{BlogID};
        require MT::Blog;
        $blog = MT::Blog->load($blog_id, {cached_ok => 1})
          or return
          $app->error(
                     MT->translate(
                                   "Load of blog '[_1]' failed: [_2]", $blog_id,
                                   MT::Blog->errstr
                     )
          );
    } else {
        $blog_id = $blog->id;
    }
    my $root = $blog->archive_path;
    return $app->error(MT->translate("You did not set your Local Archive Path"))
      unless $root;
    require MT::Author;
    require MT::Builder;
    require MT::Template::Context;
    require MT::Entry;
    my $cfg   = $plugin->config_mgr("blog:$blog_id");
    my @tmpls = (                                       # room for growth
        {
         id            => $cfg->ArchiveTemplateID,
         filename_tmpl => $cfg->FilenameTemplate
        }
    );
    my $t = {type => 1};                                # author, not commenter
    $t->{id} = $param{AuthorID} if $param{AuthorID};
    my $join =
      {'join' => ['MT::Permission', 'author_id', {blog_id => $blog_id}]};
    my $iter = MT::Author->load_iter($t, $join);
    my $builder = MT::Builder->new;
    while (my $author = $iter->()) {
        foreach my $tmpl (@tmpls) {
            my $file;
            {    # all this just to determine the file name.
                my $c = MT::Template::Context->new;
                $c->stash('blog',    $blog);
                $c->stash('blog_id', $blog_id);
                $c->stash('author',  $author);
                my $filename_tmpls =
                     MT->instance->request('__cached_filename_tmpls')
                  || MT->instance->request('__cached_filename_tmpls', {});
                my $tokens = $filename_tmpls->{$tmpl->{id}};
                unless ($tokens) {
                    $tokens = $builder->compile($c, $tmpl->{filename_tmpl})
                      or return $blog->error($builder->errstr);
                    $filename_tmpls->{$tmpl->{id}} = $tokens;
                }
                $c->stash('builder', $builder);
                $c->stash('tokens',  $tokens);
                defined($file = $builder->build($c, $tokens))
                  or return $blog->error($builder->errstr);
                $file = File::Spec->catfile($root, $file);
            }
            my $ctx = MT::Template::Context->new;
            $ctx->stash('blog',    $blog);
            $ctx->stash('blog_id', $blog_id);
            $ctx->stash('author',  $author);
            my $tmpl_id = $tmpl->{id};
            rebuild_file($app, $blog, $author, $tmpl_id, $file, $ctx)
              or die $app->errstr;
        }
    }
}

sub rebuild_file {
    my $app = shift;
    my ($blog, $author, $tmpl_id, $file, $ctx, $cond) = @_;
    my $at          = 'Author';
    my $fmgr        = $blog->file_mgr;
    my $cached_tmpl = MT->instance->request('__cached_templates')
      || MT->instance->request('__cached_templates', {});
    my $tmpl = $cached_tmpl->{$tmpl_id};
    require MT::Template;
    unless ($tmpl) {
        $tmpl = MT::Template->load($tmpl_id, {cached_ok => 1}) or return;
        $cached_tmpl->{$tmpl->id} = $tmpl;
    }
    ($file) = $file =~ /(.+)/s;    # untaint.
    return 1
      unless (
              MT->run_callbacks(
                                'BuildFileFilter',
                                Context     => $ctx,
                                ArchiveType => $at,
                                Template    => $tmpl,
                                Blog        => $blog,
                                Author      => $author,
                                File        => $file
              )
      );
    my $html = $tmpl->build($ctx, $cond);
    return $app->error($ctx->errstr) unless defined $html;
    my $orig_html = $html;
    MT->run_callbacks(
                      'BuildPage',
                      Context     => $ctx,
                      ArchiveType => $at,
                      Template    => $tmpl,
                      Blog        => $blog,
                      Author      => $author,
                      BuildResult => \$orig_html,
                      Content     => \$html,
                      File        => $file
    );
    return 1 unless $fmgr->content_is_updated($file, \$html);
    my $path = File::Basename::dirname($file);
    $path =~ s!/$!!;    ## OS X doesn't like / at the end in mkdir().
    unless ($fmgr->exists($path)) {
        $fmgr->mkpath($path)
          or return
          $app->trans_error("Error making path '[_1]': [_2]",
                            $path, $fmgr->errstr);
    }
    my $use_temp_files = !$app->config->NoTempFiles;
    my $temp_file = $use_temp_files ? "$file.new" : $file;
    defined($fmgr->put_data($html, $temp_file))
      or return
      $app->trans_error("Writing to '[_1]' failed: [_2]",
                        $temp_file, $fmgr->errstr);
    if ($use_temp_files) {
        $fmgr->rename($temp_file, $file)
          or return
          $app->trans_error("Renaming tempfile '[_1]' failed: [_2]",
                            $temp_file, $fmgr->errstr);
    }
    MT->run_callbacks(
                      'BuildFile',
                      Context     => $ctx,
                      ArchiveType => $at,
                      Template    => $tmpl,
                      Blog        => $blog,
                      Author      => $author,
                      BuildResult => \$orig_html,
                      Content     => \$html,
                      File        => $file
    );
    1;
}

1;

__END__

=cut

=head1 NAME

Cuttings.App - a plugin for publishing author archives pages
in Movable Type weblogs.

=head1 DESCRIPTION

Cuttings.App is a plugin for publishing author archives
pages in Movable Type weblogs. A cuttings file is newspaper
parlance for a collection of articles.

Despite its ability to support multiple authors, Movable
Type has never supported archives by authors. In the past
implementing a list of entries meant manually creating an
index template for each author. Besides being labourous it
also meant each and every author archive page would be
rebuilt for I<any> entry published. This contributes to
sluggish response times and unnecessary overhead to the most
common operation of any weblog tool. Depending on the number
of authors for a weblog this could make implementing these
type of archives downright impossible.

Clippings.App aims to better address the publishing of
author archives by implementing archive publishing logic
akin to that found in MT for its standard archives in
addition to working with author data outside of the context
of an entry.

The generation of other archive types and the rebuilding 
process in MT is under-developed and generally not open to
extension. Clippings.App makes the most of what MT can do
today without modifying the base code. It's implementation 
is effective though somewhat imperfect.

=head2 CAVEATS

As mentioned the implementation of author archives is
effective though somewhat imperfect. There are some rough
spots that were out of control unless we were to modify 
MT's code itself. This was thought to requires too much
expertise and was too dangerous to be considered worthwhile
for the relative minor issues they would correct. These 
issues are:

=over

=item * Author archives are I<not> generated when the
Rebuild All option is used.

=item * Not so user friendly status message while author
archives are being rebuilt. The use of
'rebuild_author_archives_only' is out of our control.

=item * The Rebuild Author Archives Only option is always
displayed even if they have not been enabled or setup.

=item * Sorting of author lists by last name or display name
is not possible.

=back

In the future we may find some ways to address these
shortcoming or perhaps Six Apart will address these
shortcomings in a future release.

=head1 TAGS

=over

=item MTAuthorArchiveURL

The absolute URL to the author's archive page.

=item MTAuthorDisplayName

The display name of the author.

=item MTAuthorEmail [spam_protect="1"]

The email address of the author. The C<spam_protect> global
filter may be used.

=item MTAuthorEntries [lastn="" offset="" sort_by=""
sort_order=""]

A container tag which iterates over a list of the entries
the author has published in the weblog.

=over

=item * lastn

Display the last N posts of the weblog. N is a positive
integer.

=item * offset

Used in coordination with lastn, starts M entries from the
start of the list. N is a positive integer.

=item * sort_by

Specifies the field to sort the list by. Recognized values
are "title," "modified_on," "author_id," and "excerpt." The
default is the authored on date.

=item * sort_order

Specifies the sort order. Recognized values are "ascend" and
"descend."

=back

=item MTAuthorEntryCount

The number of entries the author has published in the
weblog.

=item MTAuthorLink [show_email="" show_url=""]

A linked version of the author's username, using the author
URL if provided in the author's profile. Otherwise, the
author name is unlinked. This behavior can be altered with
optional attributes:

=over

=item * show_email

Specifies if the author's email can be displayed. The
default is false (0).

=item * show_url

Specifies if the author's URL can be displayed. The default
is true (1).

=back

=item MTAuthors [sort_by="" sort_order=""]

A container tag which iterates over a list of authors for
the weblog.

=over

=item * sort_by

Specifies the field to sort the list by. Recognized values
are "name," "email," and "created_on." The default is the
author username.

=item * sort_order

Specifies the sort order. Recognized values are "ascend" and
"descend." The default is "ascend."

=back

=item MTAuthorURL

The URL of the author.

=item MTAuthorUsername

The username of the author.

=back

=head1 CALLBACKS

Clippings.App supports a number of callback hooks that are 
implemented within the standard built-in publishing options. What 
differs is the information that is passed to the callback routines
as a HASH.

=over

=item * BuildFileFilter

    Context         A reference to the context object being used.
    ArchiveType     The archive type. This plugin always uses 'Author'
    Template        The template object being used to generate the file
    Blog            The blog object being used to generate the file
    Author          The author object being used to generate the file.
    File            The absolute path to be written.

=item * BuildPage

=item * BuildFile

These two callback hooks are passed the same information
except for two addition specifiers -- BuildResult and
Content.

    Content         a SCALAR reference to the file contents
    BuildResult     a SCALAR reference to a *copy* of the file contents

The difference is BuildPage is called before writing the
content to the filesystem where BuildFile is called after.
(Remember if the identical file is a duplicate to the
existing file it is not overwritten.)

=back

=head1 SETUP

Once you've installed Clippings.App on your system you are
ready to design a author archives template. For each weblog
you want to enable with author archives you'll need to take
the following steps.

=over

=item 1 Create a new archive template. 

Go into the Archives tab in Templates and click the "Create
New Archive Template" link. Design the template using any of
the MTAuthor tags provided by the plugin or any of the
built-in MTBlog tags. Save your template once created.

=item 2 Map the template and define a file naming
template.

Once you've saved your archive template go to the Plugin
Settings tabs in your weblog. Go down to the Clippings.App
listing and click the "Show Settings" link. This will
display a pulldown menu with all of the defined archive
templates in your system. Select your author archive
template. You may also control the naming and placement of
the published files using MTAuthor and MTBlog tags in the
Filename Template text box. The default is the author
username with the default file extension that will be
written into the weblog's root directory. Save your
settings.

=item 3 Publish your author archive files.

Select Rebuild Site from the Weblog Toolbar. In the pulldown
menu you will find an option labeled "Rebuild Author
Archives Only." Select it and click rebuild. B<NOTE:>
Because of a limitation in MT Author Archives are B<not>
published when "Rebuild All" is selected.

=back

=head1 PUBLISHING ARCHIVES

Author archive pages can be published at any time using the
"Rebuild Author Archives Only" option in the pulldown menu
when the Rebuild Site pop-up is open. This should only be
necessary when the archives are first introduced or if the
author archive template is modified. An author's archive
page will be automatically updated each time an entry is
saved.

B<NOTE:> Because of a limitation in MT Author Archives are
B<not> published when "Rebuild All" is selected.

=head1 LICENSE

See LICENSE included in the Cuttings.App distribution.

=head1 AUTHOR & COPYRIGHT

This code cannot be redistributed without permission from
the author. Except where otherwise noted, Cuttings.App is
Copyright 2006, Timothy Appnel, Appnel Internet Solutions, 
LLC, tim@appnel.com. All rights reserved.

=cut
